//******************************************************************
//
//	パスワールド PW10164液晶時計
//
//	DEVICE:ATmega168
//	efuse:F9h	
//	hfuse:D6h	BOD 1.8V	チップ消去から EEPROM保護
//	lfuse:62h	内蔵CR発振 1MHz
//
//	Compiler:	AVR-GCC		3.4.6
//				AVR-LIBC	1.4.7
//				binutils	2.16.1
//
//	2008.5.29	スペルミス Thr -> Thu
//	2007.11.17	KAWAKAMI Yukio
//******************************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdlib.h>
#include	<string.h>
#include	<stdio.h>
#include	<ctype.h>
#include	<util/parity.h>
#include	<util/twi.h>

#include	"LCDclock.h"
#include	"LCDfont.h"

#include	"pata-base.c"
#include	"numbers.c"
#include	"broken.c"

#define		NOIZE_LEVEL		2		// ノイズレベル 1〜10
#define		ADJUST_JJY_DATE	0		// 電波時計の日付データは使わない

#define		ADJUST_HOUR		0x03
#define		ADJUST_MIN		0x17	// 毎日午前３時17分から。BCDなので16進指定
#define		ADJUST_LIMIT	15		// 最大15分間、電波時計補正を行う


PROGMEM uchar WeekStr[7][4] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

PROGMEM char MonthDays[]
	= { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

uchar	LcdBuff[101*64/8];	// LCDバッファ 808bytes
uchar	LcdContrast;		// LCDコントラスト

uchar	AutoAdjustHour;		// JJY補正する時間(時) BCDで入っていることに注意
uchar	AutoAdjustMin;		//		〃        (分)

struct RTC_TIME	JJYAdjustTime;	// JJY補正した時間

volatile uchar	SecFlag;		// 1秒割り込みフラグ

volatile uchar	Timer;
volatile uchar	AdjustMin;
volatile uchar	AdjustSec;
volatile uchar	AdjustDS;

volatile uchar	JJYok;			// JJY受信状態
volatile uchar	JJYbuff[8];		// 電波時計受信データバッファ
volatile uchar	JJYflag;		// JJY信号立ち上がりフラグ

volatile uchar	Debug;

//*******************************************************
//	タイマー２比較一致A割り込み
//	1MHz / 64 / 156 = 100Hz
//*******************************************************
ISR(TIMER2_COMPA_vect) {
	static uchar	state;
	static uchar	bit;
	static uchar	interval;
	static uchar	width;
	static uchar	width_flag;

	if (Timer)	--Timer;

	// 電波時計補正制限時間タイマ
	if (AdjustMin){
		if (++AdjustDS >= 100){
			AdjustDS = 0;
			if (++AdjustSec >= 60){
				AdjustSec = 0;
				--AdjustMin;
			}
		}
	}
	
	if (CHECK_JJY_ON){	// JJYが有効になっていれば
		uchar	p;

		// 電波時計の信号処理
		if (JJY_ON){
			p = 1;
		} else {
			p = 0;
		}

		if (interval < 255)	interval++;
		
		if (state == 0){
			// 信号立ち上がり検出
			if (p){
				bit = 0;
				state++;
				if ((interval >= 98)&&(interval <= 102)){
					// １秒毎に信号が来ている
					if (JJYok < 4)	JJYok++;
				}
				interval = 0;
				width = 0;
				width_flag = 1;
			}
		} else if (state < 90){
			state ++;
			if (p){
				if (width_flag)	width++;	// 信号の幅
				if ((state >= 55)&&(state < 82)){
					bit++;			// 550〜820の間で信号があるか
				}
			} else {
				width_flag = 0;		// 信号が途切れた
			}
		} else if (state == 90) {
			uchar	i;

			state++;

			// 信号幅から受信状態を推測
			if ((width < 15)||
				((width > 25)&&(width < 45))||
				((width > 55)&&(width < 75))||
				(width > 85)){
				if (JJYok > 0)	--JJYok;
			}
			
			// データをシフト
			for (i = 0; i < 7; i++){
				JJYbuff[i] <<= 1;
				JJYbuff[i] |= (JJYbuff[i+1] >> 7);
			}
			JJYbuff[7] <<= 1;
			if (bit < NOIZE_LEVEL){
				JJYbuff[7] |= 0x10;		// 1
			}
			// マスクと比較して同期判定
			if (((JJYbuff[0] & 0x88) == 0x80)&&
				((JJYbuff[1] & 0x72) == 0x40)&&
				((JJYbuff[2] & 0x1C) == 0x10)&&
				((JJYbuff[3] & 0x84) == 0x04)&&
				((JJYbuff[4] & 0x31) == 0x01)&&
				((JJYbuff[6] & 0x41) == 0x40)&&
				((JJYbuff[7] & 0xF0) == 0x10)){
				uchar	data;

				JJYflag = 1;	// メインルーチンでデコードする

				// 分パリティチェック
				data = (JJYbuff[0] & 0x70)|((JJYbuff[0] & 0x07)<<1)|(JJYbuff[1]>>7);
				data = parity_even_bit(data);
				data ^= ((JJYbuff[4] & 0x04)>>2);
				if (data)	JJYflag = 0;	// パリティエラー

				// 時パリティチェック
				data = ((JJYbuff[1] & 0x0C)<<2)|((JJYbuff[1] & 0x01)<<3)|((JJYbuff[2] & 0xE0)>>5);
				data = parity_even_bit(data);
				data ^= ((JJYbuff[4] & 0x08)>>3);
				if (data)	JJYflag = 0;	// パリティエラー
			}
		} else {
			// 信号OFF待ち
			if (!p){
				state = 0;
			}
		}
	}
}


//*******************************************************
//	INT0 割り込み
//	RTCからのクロックで１秒毎に発生
//*******************************************************
ISR(INT0_vect) {

	SecFlag = 1;
}


//*******************************************************
//	INT1 割り込み
//	SW1が押された時に発生
//*******************************************************
ISR(INT1_vect) {
}



//*******************************************************
//	メインルーチン
//*******************************************************
int main (void) {
	struct RTC_TIME	rtc;
	uchar	buff[20];
	uchar	day;
	uchar	min;

	cli();
	
	PORTB = (1<<7)|(1<<6)|(1<<SPI_CLK)|(1<<SPI_DOUT)|(1<<LCDCS);
	DDRB = (1<<SPI_CLK)|(1<<SPI_DOUT)|(1<<LCD_DC)|(1<<LCDCS)|(1<<LCDRS);

	PORTC = (1<<7)|(1<<6)|(1<<SCL)|(1<<SDA)|(1<<1)|(1<<0);
	DDRC = (1<<JJY_PON);

	PORTD = (1<<7)|(1<<6)|(1<<5)|(1<<SW2)|(1<<SW1)|(1<<1)|(1<<0);
	DDRD = 0;

	// CPUクロックを 1MHzにする
	CLKPR = (1<<CLKPCE);
	CLKPR = (1<<CLKPS1)|(1<<CLKPS0);

	// タイマ 0, 1、USART電力カット
	PRR = (1<<PRTIM1)|(1<<PRTIM0)|(1<<PRUSART0);
	ACSR = (1<<ACD);			// アナログコンパレータ禁止

	// タイマ2
	TCCR2A = (1<<WGM21);	// CTCモード
	TCCR2B = (1<<CS22);		// プリスケーラ 1/64  1MHz/64/156≒100Hz
	OCR2A = (156 - 1);
	TCNT2 = 0;
	TIMSK2 = (1<<OCIE2A);	// 比較A割り込み許可

	// INT0 立ち上がり、INT1 立ち下がりエッジ割り込み
	EICRA = (1<<ISC11)|(1<<ISC01)|(1<<ISC00);
	EIMSK = (1<<INT1)|(1<<INT0);	// 割り込み許可
	
	// SPIマスタ, CPOL=1, CPHA=1, Clock 1MHz/4=250kHz
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA);
	SPSR = (1<<SPI2X);	// 1/2 500kHz

	MCUCR = (1<<SE);		// スリープ許可 アイドルモード

	sei();		// 割り込み許可

	// 電源等、安定化するまで待つ
	Timer = 10;
	while(Timer)	SLEEP;

	// JJY自動補正時刻読み込み
	AutoAdjustHour = eeprom_read_byte((uint8_t *)EEPROM_ADJUST_HOUR);
	if (AutoAdjustHour > 0x23)	AutoAdjustHour = 0x03;		// デフォルト３時
	AutoAdjustMin = eeprom_read_byte((uint8_t *)EEPROM_ADJUST_MIN);
	if (AutoAdjustMin > 0x59)	AutoAdjustMin = 0x17;		// デフォルト 17分

	//LcdContrast = eeprom_read_byte((uint8_t *)EEPROM_CONTRAST);
	//if (LcdContrast > 63)	LcdContrast = 50;	// 初期値
	LcdContrast = 32;
	
	LCD_init();					// LCD初期化
	LCD_contrast(LcdContrast);	// コントラスト設定
	print_P(0, 0, PSTR("initializing..."));

	// TWIリセット?
	TWI_open();
	TWI_write(0xFF);
	TWI_close();
	
	// RTC設定 これを設定しないと INT1割り込みが動作しない
	TWI_open();
	TWI_write(0xA2);	// スレーブアドレス + Write
	TWI_write(0);		// レジスタ 0(Control1)
	TWI_write(0);		// クロック動作
	TWI_close();

	TWI_open();
	TWI_write(0xA2);	// スレーブアドレス + Write
	TWI_write(0x0D);	// レジスタ 0Dh(CLKOUT frequency)
	TWI_write(0x83);	// CLKOUT = 1Hz
	TWI_close();

	// 起動時の電波時計補正
	AdjustMin = 10;		// 10分

	day = 99;
	min = 99;
	SecFlag = 0;
	for (;;){
		put_wallpaper(GraphData);
		for (;;){
			WDR;
			read_RTC(&rtc);
			if (min != rtc.min){
				// 分が更新された
				clear_flip();
				if (rtc.hour >> 4){
					put_graph_num(6, &NumData[rtc.hour >> 4]);
				}
				put_graph_num(26, &NumData[rtc.hour & 0x0F]);
				put_graph_num(56, &NumData[rtc.min >> 4]);
				put_graph_num(76, &NumData[rtc.min & 0x0F]);
				trans_flip();
				min = rtc.min;
				if ((rtc.hour == AutoAdjustHour)&&
					(rtc.min == AutoAdjustMin)){
					start_adjust();		// 電波時計補正時間が来た
				}
			}
			if (day != rtc.day){
				// 日付が更新された
				sprintf_P(buff, PSTR("20%02X/%02X/%02X"),
						rtc.year, rtc.month, rtc.day);
				print(1, 0, buff);
				print_P(12, 0, &WeekStr[rtc.week]);
				day = rtc.day;
			}
			// 時：分：秒 表示
			sprintf_P(buff, PSTR("%02X:%02X:%02X"),
					rtc.hour, rtc.min, rtc.sec);
			print_shift(8, 7, buff);

			if (!CHECK_JJY_ON){
				// CPUクロックを 32kHzに落としてスリープ
				CLKPR = (1<<CLKPCE);
				CLKPR = (1<<CLKPS3);	// 1/256
				SLEEP;

				// CPUクロックを 1MHzに戻す
				CLKPR = (1<<CLKPCE);
				CLKPR = (1<<CLKPS1)|(1<<CLKPS0);
			} else if (AdjustMin == 0){
				// 電波時計補正時間タイムアウト
				print_P(0, 7, PSTR("   "));	// 時刻同期時の状態表示消去
				cli();
				JJY_DISABLE;
				sei();
				PRR |= (1<<PRTIM2);		// タイマー２電源OFF
			} else {
				if (JJY_adjusting()){	// 時刻同期するか、秒が更新されたら戻る
					// 同期成功
					AdjustMin = 0;		// JJY同期タイマクリア
				}
				SecFlag = 0;
			}

			if (CHECK_SW1){
				menu();			// メニュー
				JJYflag = 0;	// 念のため
				day = 99;
				min = 99;
				break;
			}
		}
	}
}


//*******************************************************
//	電波時計で時刻同期開始
//*******************************************************
void start_adjust(void){
	cli();
	AdjustMin = ADJUST_LIMIT;
	JJY_ENABLE;
	JJYok = 0;
	JJYbuff[6] = 0;
	JJYbuff[7] = 0;
	PRR &= ~(1<<PRTIM2);		// タイマー２電源ON
	sei();
}


//*******************************************************
//	JJY時刻同期動作
//	時刻同期完了するか、あるいは RTCの秒が更新されたら戻る
//*******************************************************
uchar JJY_adjusting(void){
	uchar	old_jjy;
	uchar	old_ok;
	uchar	j;
	uchar	r;

	// JJY受信モード
	old_jjy = 99;
	old_ok = 0;
	r = 0;
	while(!SecFlag){
		WDR;
		SLEEP;

		if (CHECK_SW1)	break;	// スイッチ１が押されたら抜ける

		j = JJY_ON;
		if (j != old_jjy){
			if (j){
				print_shift_P(0, 7, PSTR("\x06"));	// ●
			} else {
				print_P(0, 7, PSTR(" "));
			}
			old_jjy = j;
		}
		if (JJYok != old_ok){
			switch(JJYok){
			  case 1:
				print_shift_P(1, 7, PSTR("\x01\x02"));	break;	// アンテナ
			  case 2:
				print_shift_P(1, 7, PSTR("\x01\x03"));	break;	// 1本
			  case 3:
				print_shift_P(1, 7, PSTR("\x01\x04"));	break;	// 2本
			  case 4:
				print_shift_P(1, 7, PSTR("\x01\x05"));	break;	// 3本
			  default:
				print_P(1, 7, PSTR("  "));	break;	// なし
			}
			old_ok = JJYok;
		}

		if (JJYflag){
			struct RTC_TIME	jjy;
			uchar	i;

			// 電波時計の時刻が取得できた この段階で 59秒台

			JJYflag = 0;

#if ADJUST_JJY_DATE == 0
			read_RTC(&jjy);	// 現在のRTCの日付を保持
#endif
			JJY2RTC(&jjy);	// RTCレジスタ型式に変換

			// RTC補正
			WDR;
			next_min(&jjy);	// 1分進める
			for (i = 0; i < 20; i++){
				if (JJY_ON)	break;
				SLEEP;
			}
			adjust_RTC(&jjy);			// RTCに値をセット
			JJYAdjustTime = jjy;		// 補正した日時を保存
			r = 1;						// 時刻同期成功
		}
	}

	return r;
}


//*******************************************************
//	ボタンメニュー
//*******************************************************
void menu(void){
	uchar	prr;
	uchar	m;
	uchar	key;
	uchar	time_out;

	prr = PRR;				// 電力管理レジスタ保存
	PRR &= ~(1<<PRTIM2);	// タイマー２動作開始
	EIMSK &= ~(1<<INT1);	// SW1割り込み禁止

	m = 0;
	key = 0;
	while(key != ESC){
		clear_LCD();
		print_P(1, 0, PSTR("Quit"));
		print_P(1, 1, PSTR("View Prev. Adj."));
		print_P(1, 2, PSTR("Manual Adjust"));
		print_P(1, 3, PSTR("Set Adj. Time"));
		print_P(1, 4, PSTR("Broken Demo"));
		print_P(0, m, PSTR("#"));
		wait_switch_off();		// SW1, 2 が OFFになるまで待つ
		time_out = 0;
		Timer = 100;
		while(!CHECK_SW1 && !CHECK_SW2){
			WDR;
			SLEEP;
			if (!Timer){
				if (++time_out >= 15){
					goto QUIT;
				} else {
					Timer = 100;
				}
			}
		}
		if (CHECK_SW2){
			if (++m > 4)	m = 0;
		} else {
			switch(m){
			  case 0:
				key = ESC;
				break;
			  case 1:
				view_adjust_time();		// 前回時刻補正した時間を表示
				break;
			  case 2:
				adjust_clock();			// マニュアルで時間合わせ
				if (AdjustMin){
					prr &= ~(1<<PRTIM2);	// タイマー２電源ON
				}
				break;
			  case 3:
				set_adjust_time();		// JJY自動調整時間セット
				break;
			  case 4:
				broken_demo();			// 電脳空間が壊れたデモ
				break;
			}
		}
	}

  QUIT:
	wait_switch_off();		// SW1, 2 が OFFになるまで待つ
	PRR = prr;				// 電力管理レジスタを戻す
	EIFR = (1<<INTF1);		// SW1割り込みフラグクリア
	EIMSK |= (1<<INT1);		// SW1割り込み許可
}


//*******************************************************
//	電脳コイルの「電脳空間が壊れた」みたいな表示デモ
//*******************************************************
void broken_demo(void){
	struct RTC_TIME	rtc;
	uchar	buff[20];
	uchar	x, y;
	uint	i;
	uchar	*p;

	put_wallpaper(GraphData);
	read_RTC(&rtc);
	if (rtc.hour >> 4){
		put_graph_num(6, &NumData[rtc.hour >> 4]);
	}
	put_graph_num(26, &NumData[rtc.hour & 0x0F]);
	put_graph_num(56, &NumData[rtc.min >> 4]);
	put_graph_num(76, &NumData[rtc.min & 0x0F]);
	trans_flip();
	sprintf_P(buff, PSTR("20%02X/%02X/%02X"),
			rtc.year, rtc.month, rtc.day);
	print(1, 0, buff);
	print_P(12, 0, &WeekStr[rtc.week]);
	sprintf_P(buff, PSTR("%02X:%02X:%02X"),
			rtc.hour, rtc.min, rtc.sec);
	print_shift(8, 7, buff);

	print_P(30/6, 32/8, PSTR("        "));
	for (i = 0; i < 101*64/8; i++){
		LcdBuff[i] |= (uchar)pgm_read_byte(&BrokenData[i]);
	}

	p = LcdBuff;
	for (y = 0; y < 8; y++){
		LCD_set(0, y);
		for (x = 0; x < 101; x++){
			LCD_data(*p++);
		}
	}

	wait_switch_off();		// SW1, 2 が OFFになるまで待つ

	while(!CHECK_SW2){
		WDR;
		SLEEP;
	}

	wait_switch_off();		// SW1, 2 が OFFになるまで待つ
}


//*******************************************************
//	前回電波時計で補正した日時を表示する
//*******************************************************
void view_adjust_time(void){
	uchar	buff[20];
	uchar	time_out;

	clear_LCD();
	print_P(0, 0, PSTR("Prev. Adjust Time"));
	if (JJYAdjustTime.year == 0){
		print_P(0, 1, PSTR("Not Adjust"));
	} else {
		sprintf_P(buff, PSTR("20%02X/%02X/%02X"),
				JJYAdjustTime.year, JJYAdjustTime.month, JJYAdjustTime.day);
		print(0, 1, buff);
		sprintf_P(buff, PSTR("%02X:%02X:00"),
				JJYAdjustTime.hour, JJYAdjustTime.min);
		print(0, 2, buff);
	}

	wait_switch_off();		// SW1, 2 が OFFになるまで待つ

	time_out = 0;
	Timer = 100;
	while(!CHECK_SW2){
		WDR;
		SLEEP;
		if (!Timer){
			if (++time_out >= 15)	break;
			Timer = 100;
		}
	}

	wait_switch_off();		// SW1, 2 が OFFになるまで待つ
}


//*******************************************************
//	JJY自動補正時刻の変更
//*******************************************************
void set_adjust_time(void){
	uchar	buff[20];
	uchar	h, m;

	h = AutoAdjustHour;
	m = AutoAdjustMin;

	clear_LCD();
	print_P(0, 0, PSTR("Set Adjust Time"));
	sprintf_P(buff, PSTR("%02X:%02X"),
			  h, m);
	print(1, 1, buff);

	if (adjust_clock_sub(1, 1, &h, 0, 23) == ESC)	goto QUIT;
	if (adjust_clock_sub(4, 1, &m, 0, 59) == ESC)	goto QUIT;

	eeprom_write_byte((uint8_t *)EEPROM_ADJUST_HOUR, h);
	eeprom_write_byte((uint8_t *)EEPROM_ADJUST_MIN, m);
	AutoAdjustHour = h;
	AutoAdjustMin = m;
  QUIT:
	return;
}


//*******************************************************
//	時計合わせ
//*******************************************************
void adjust_clock(void){
	struct RTC_TIME	rtc;
	uchar	buff[20];
	int		y, m;
	
	clear_LCD();
	print_P(0, 0, PSTR("Adjust clock mode"));
	read_RTC(&rtc);
	sprintf_P(buff, PSTR("20%02X/%02X/%02X"),
			rtc.year, rtc.month, rtc.day);
	print(0, 1, buff);
	sprintf_P(buff, PSTR("%02X:%02X:00"),
			rtc.hour, rtc.min);
	print(0, 2, buff);

	if (adjust_clock_sub(2, 1, &(rtc.year), 0, 99) == ESC)	goto QUIT;
	if (adjust_clock_sub(5, 1, &(rtc.month), 1, 12) == ESC)	goto QUIT;
	if (adjust_clock_sub(8, 1, &(rtc.day), 1, 31) == ESC)	goto QUIT;
	if (adjust_clock_sub(0, 2, &(rtc.hour), 0, 23) == ESC)	goto QUIT;
	if (adjust_clock_sub(3, 2, &(rtc.min), 0, 59) == ESC)	goto QUIT;

	// 日付から曜日を求める
	m = BIN(rtc.month);
	y = BIN(rtc.year) + 2000;
	if (m <= 2){
		--y;
		m += 12;
	}
	rtc.week = (BIN(rtc.day)
				+ ((8 + (13 * m)) / 5)
				+ (y + (y / 4) - (y / 100) + (y / 400))) % 7;

	rtc.sec = 0;	// 00秒

	adjust_RTC(&rtc);		// RTC設定

	wait_switch_off();		// SW1, 2 が OFFになるまで待つ

	start_adjust();			// JJY同期開始
  QUIT:
	return;
}


//*******************************************************
//	時計合わせ　サブ
//*******************************************************
uchar adjust_clock_sub(uchar x, uchar y, uchar *n, uchar min, uchar max){
	uchar	buff[3];
	uchar	data;
	uchar	timeout;
	uchar	repeat;

	wait_switch_off();		// SW1, 2 が OFFになるまで待つ

	data = BIN(*n);
	repeat = 0;
	for (;;){
		WDR;
		sprintf_P(buff, PSTR("%02d"), data);
		print(x, y, buff);
		Timer = 100;
		timeout = 0;
		while((!CHECK_SW1)&&(!CHECK_SW2)){
			WDR;
			SLEEP;
			if ((Timer == 20)||(Timer == 70)){
				print_P(x, y, PSTR("\x7F\x7F"));
			} else if (Timer == 50){
				print(x, y, buff);
			} else if (Timer == 0){
				if (++timeout >= 15)	return ESC;	// 15秒放置でタイムアウト
				Timer = 100;
				print(x, y, buff);
			}
			repeat = 0;
		}
		if (CHECK_SW1){
			print(x, y, buff);
			*n = BCD(data);
			return CR;
		} else if (CHECK_SW2){
			if (repeat == 0){
				if (++data >= max){
					data = min;
				}
				repeat++;
			} else if (repeat == 100){
				if (++data > max){
					data = min;
				}
				repeat = 50;
			} else {
				repeat++;
			}
			Timer = 1;
			while(Timer)	SLEEP;	// 10ms
		}
	}
}


//*******************************************************
//	壁紙(101x64ドット画像)表示
//*******************************************************
void put_wallpaper(PGM_VOID_P graph){
	uchar	x, y;
	uchar	d, *p;

	p = LcdBuff;
	for (y = 0; y < 8; y++){
		LCD_set(0, y);
		for (x = 0; x < 101; x++){
			d = pgm_read_byte(graph++);
			LCD_data(d);
			*p++ = d;
		}
	}
}


//*******************************************************
//	JJYバッファの内容を RTCレジスタ型式に変換
//*******************************************************
void JJY2RTC(struct RTC_TIME *rtc){
#if ADJUST_JJY_DATE
	uint	day;
#endif

	rtc->sec = 0;
	rtc->min = (JJYbuff[0] & 0x70)|((JJYbuff[0] & 0x07)<<1)|(JJYbuff[1]>>7);
	rtc->hour = ((JJYbuff[1] & 0x0C)<<2)|((JJYbuff[1] & 0x01)<<3)|((JJYbuff[2] & 0xE0)>>5);

#if ADJUST_JJY_DATE
	day = (JJYbuff[2] & 0x03) * 100
		+ ((JJYbuff[3] & 0x78)>>3) * 10
		+ ((JJYbuff[3] & 0x03)<<2)
		+ ((JJYbuff[4] & 0xC0)>>6);
	rtc->year = ((JJYbuff[5] & 0x7F)<<1)|((JJYbuff[6] & 0x80)>>7);
	for(i = 1; i < 12; i++){
		uchar	last_day, i;
		last_day = (char)pgm_read_byte(&MonthDays[i]);
		if ((i == 2)&&((BIN(rtc->year) % 4) == 0)){
			last_day++;
		}
		if (day <= last_day){
			break;
		} else {
			day -= last_day;
		}
	}
	rtc->month = BCD(i);
	rtc->day = BCD(day);
	rtc->week = (JJYbuff[6] & 0x38) >> 3;
#endif
}


//*******************************************************
//	RTCレジスタバッファの内容を１分進める
//*******************************************************
void next_min(struct RTC_TIME *rtc){
	uchar	last_day;

	rtc->min = add_BCD(rtc->min, 1);
	if (rtc->min >= 0x60){
		rtc->min = 0;
		rtc->hour = add_BCD(rtc->hour, 1);
		if (rtc->hour >= 0x24){
			rtc->hour = 0;
			rtc->day = add_BCD(rtc->day, 1);
			last_day = (char)pgm_read_byte(&MonthDays[BIN(rtc->month)]);
			if ((rtc->month == 2)&&((BIN(rtc->year) % 4) == 0)){
				last_day++;
			}
			last_day = ((last_day / 10) << 4)|(last_day % 10);
			if (rtc->day > last_day){
				rtc->day = 1;
				rtc->month = add_BCD(rtc->month, 1);
				if (rtc->month >= 0x13){
					rtc->month = 1;
					rtc->year = add_BCD(rtc->year, 1);
				}
			}
		}
	}
}


//*******************************************************
//	BCD加算
//*******************************************************
uchar add_BCD(uchar source, uchar add){

	source += add;
	if ((source & 0x0F) >= 0x0A)	source += 6;
	if (source >= 0xA0)				source += 0x60;
	return source;
}


//*******************************************************
//	SW1, 2が両方 OFFになるまで待つ
//*******************************************************
void wait_switch_off(void){

	while(CHECK_SW1||CHECK_SW2){	// SW1, 2 が OFFになるまで待つ
		WDR;
		Timer = 2;
		while(Timer)	SLEEP;
	}
}


//*******************************************************
//	BCD → バイナリ変換
//*******************************************************
uchar BIN(uchar bcd){

	return (bcd >> 4) * 10 + (bcd & 0x0F);
}


//*******************************************************
//	バイナリ → BCD変換
//*******************************************************
uchar BCD(uchar bin){

	return ((bin / 10) << 4) | (bin % 10);
}


//****************************************
//	SPIデータ出力
//****************************************
void SPI_out(uchar data){

	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
}


//****************************************
//	SPIデータ入力
//****************************************
uchar SPI_in(void){

	SPDR = 0xFF;
	while(!(SPSR & (1<<SPIF)));
	return SPDR;
}


//*******************************************************
//	Clock moduleから時間読み込み
//*******************************************************
void read_RTC(struct RTC_TIME *rtc){

	TWI_open();
	TWI_write(0xA2);	// スレーブアドレス + Write
	TWI_write(2);		// レジスタ 2(Second)
	TWI_start();		// 開始条件
	TWI_write(0xA3);	// スレーブアドレス + Read
	rtc->sec = TWI_read_ack();
	rtc->min = TWI_read_ack() & 0x7F;
	rtc->hour = TWI_read_ack() & 0x3F;
	rtc->day = TWI_read_ack() & 0x3F;
	rtc->week = TWI_read_ack() & 0x07;
	rtc->month = TWI_read_ack() & 0x1F;
	rtc->year = TWI_read_nack();
	TWI_close();

	// VLビットチェック
	if (rtc->sec & 0x80){
		rtc->year = 0;		// 2000/01/01
		rtc->month = 1;
		rtc->day = 1;
		rtc->sec &= 0x7F;	// VLビットクリア
		// 時・分・秒は残す
	}
}


//*******************************************************
//	RTC時計合わせ
//*******************************************************
void adjust_RTC(struct RTC_TIME *rtc){

	TWI_open();
	TWI_write(0xA2);	// スレーブアドレス + Write
	TWI_write(0);		// レジスタ 0(Control1)
	TWI_write(0x20);	// クロック動作停止
	TWI_close();

	TWI_open();
	TWI_write(0xA2);	// スレーブアドレス + Write
	TWI_write(2);		// レジスタ 2(Seconds)
	TWI_write(rtc->sec);
	TWI_write(rtc->min);
	TWI_write(rtc->hour);
	TWI_write(rtc->day);
	TWI_write(rtc->week);
	TWI_write(rtc->month);
	TWI_write(rtc->year);
	TWI_close();

	TWI_open();
	TWI_write(0xA2);	// スレーブアドレス + Write
	TWI_write(0);		// レジスタ 0(Control1)
	TWI_write(0);		// クロック動作
	TWI_close();
}


//*******************************************************
// Clock module TWI Open
//*******************************************************
char TWI_open(void){
	uint	i;

	TWBR = 0;
	TWSR = 0;		// TWI clock 62.5kHz

	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);	// 開始条件送出
	for (i = 0; i < 10000; i++){
		if (TWCR & (1<<TWINT))	break;
	}

	if ((TWSR & TW_NO_INFO) != TW_START){
		return 1;	// ERROR
	} else {
		return 0;
	}
}


//*******************************************************
//	TWI Write
//*******************************************************
char TWI_write(uchar data){
	uint	i;

	TWDR = data;	// データ
	TWCR = (1<<TWINT)|(1<<TWEN);
	for (i = 0; i < 10000; i++){
		if (TWCR & (1<<TWINT))	break;
	}

	if ((TWSR & TW_NO_INFO) != TW_MT_DATA_ACK){
		return 1;	// ERROR
	} else {
		return 0;
	}
}


//*******************************************************
// TWI Start
//*******************************************************
char TWI_start(void){
	uint	i;

	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);	// 開始条件送出

	for (i = 0; i < 10000; i++){
		if (TWCR & (1<<TWINT))	break;
	}

	if ((TWSR & TW_NO_INFO) != TW_REP_START){
		return 1;	// ERROR
	} else {
		return 0;
	}
}


//*******************************************************
//	TWI Read ACK
//*******************************************************
uchar TWI_read_ack(void){
	uint	i;

	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);

	for (i = 0; i < 10000; i++){
		if (TWCR & (1<<TWINT))	break;
	}

	if ((TWSR & TW_NO_INFO) != TW_MR_DATA_ACK){
		return 0;		// ERROR
	} else {
		return TWDR;	// データ
	}
}


//*******************************************************
//	TWI Read NACK
//*******************************************************
uchar TWI_read_nack(void){
	uint	i;

	TWCR = (1<<TWINT)|(1<<TWEN);

	for (i = 0; i < 10000; i++){
		if (TWCR & (1<<TWINT))	break;
	}

	if ((TWSR & TW_NO_INFO) != TW_MR_DATA_NACK){
		return 0;		// ERROR
	} else {
		return TWDR;	// データ
	}
}


//*******************************************************
//	TWI Close
//*******************************************************
void TWI_close(void){

	TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);
}


//*********************************************************
//	5*7 半角文字列出力
//	アイコンマークは 6*7ドット
//*********************************************************
void print(uchar x, uchar y, uchar *str){
	uchar	c;
	PGM_VOID_P	p;
	uchar	*bp;
	uchar	i;
	uchar	d;

	x *= 6;
	LCD_set(x, y);
	bp = &LcdBuff[y*101+x];

	while((c = *str++) != 0){
		if (c >= ' '){
			p = (PGM_VOID_P)&(AsciiFont[(c - 0x20) * 5]);
		} else {
			p = (PGM_VOID_P)&(ExtFont[(c - 1) * 6]);
		}
		for (i = 0; i < 5; i++){
			d = (uchar)pgm_read_byte(p++);
			LCD_data(d);
			*bp++ = d;
		}
		if (c < ' '){
			d = (uchar)pgm_read_byte(p++);
			LCD_data(d);
			*bp = d;
		} else {
			LCD_data(0);
			*bp = 0;
		}
	}
}


//*********************************************************
//	5*7 半角文字列出力
//*********************************************************
void print_P(uchar x, uchar y, PGM_VOID_P str){
	uchar	c;
	PGM_VOID_P	p;
	uchar	*bp;
	uchar	i;
	uchar	d;

	x *= 6;
	LCD_set(x, y);
	bp = &LcdBuff[y*101+x];

	while((c = (uchar)pgm_read_byte(str++)) != 0){
		if (c >= ' '){
			p = (PGM_VOID_P)&(AsciiFont[(c - 0x20) * 5]);
		} else {
			p = (PGM_VOID_P)&(ExtFont[(c - 1) * 6]);
		}
		for (i = 0; i < 5; i++){
			d = (uchar)pgm_read_byte(p++);
			LCD_data(d);
			*bp++ = d;
		}
		if (c < ' '){
			d = (uchar)pgm_read_byte(p++);
			LCD_data(d);
			*bp = d;
		} else {
			LCD_data(0);
			*bp = 0;
		}
	}
}


//*********************************************************
//	5*7 半角文字列出力
//	アイコンマークは 6*7ドット
//	１ドット下に表示
//*********************************************************
void print_shift(uchar x, uchar y, uchar *str){
	uchar	c;
	PGM_VOID_P	p;
	uchar	*bp;
	uchar	i;
	uchar	d;

	x *= 6;
	LCD_set(x, y);
	bp = &LcdBuff[y*101+x];

	while((c = *str++) != 0){
		if (c >= ' '){
			p = (PGM_VOID_P)&(AsciiFont[(c - 0x20) * 5]);
		} else {
			p = (PGM_VOID_P)&(ExtFont[(c - 1) * 6]);
		}
		for (i = 0; i < 5; i++){
			d = (uchar)pgm_read_byte(p++);
			d <<= 1;
			LCD_data(d);
			*bp++ = d;
		}
		if (c < ' '){
			d = (uchar)pgm_read_byte(p++);
			d <<= 1;
			LCD_data(d);
			*bp = d;
		} else {
			LCD_data(0);
			*bp = 0;
		}
	}
}


//*********************************************************
//	5*7 半角文字列出力
//	１ドット下に表示
//*********************************************************
void print_shift_P(uchar x, uchar y, PGM_VOID_P str){
	uchar	c;
	PGM_VOID_P	p;
	uchar	*bp;
	uchar	i;
	uchar	d;

	x *= 6;
	LCD_set(x, y);
	bp = &LcdBuff[y*101+x];

	while((c = (uchar)pgm_read_byte(str++)) != 0){
		if (c >= ' '){
			p = (PGM_VOID_P)&(AsciiFont[(c - 0x20) * 5]);
		} else {
			p = (PGM_VOID_P)&(ExtFont[(c - 1) * 6]);
		}
		for (i = 0; i < 5; i++){
			d = (uchar)pgm_read_byte(p++);
			d <<= 1;
			LCD_data(d);
			*bp++ = d;
		}
		if (c < ' '){
			d = (uchar)pgm_read_byte(p++);
			d <<= 1;
			LCD_data(d);
			*bp = d;
		} else {
			LCD_data(0);
			*bp = 0;
		}
	}
}


//*********************************************************
//	数字を右詰め表示
//*********************************************************
void print_num_right(uint num, uchar x, uchar y, uchar w){
	uchar	buff[6];
	uchar	l;

	utoa(num, buff, 10);
	l = strlen(buff);
	while(l < w){
		print_P(x++, y, PSTR(" "));
		l++;
	}
	print(x, y, buff);
}


//*********************************************************
//	数字を右詰めゼロ付き表示
//*********************************************************
void print_num_zero(uint num, uchar x, uchar y, uchar w){
	uchar	buff[6];
	uchar	l;

	utoa(num, buff, 10);
	l = strlen(buff);
	while(l < w){
		print_P(x++, y, PSTR("0"));
		l++;
	}
	print(x, y, buff);
}


//*********************************************************
//	LcdBuff[]に数字グラフィックセット
//*********************************************************
void put_graph_num(uchar xs, PGM_VOID_P graph){
	uchar	x, y;
	uchar	*p;

	for (y = 0; y < (32 / 8); y++){
		p = &LcdBuff[(y + 16 / 8) * 101 + xs];
		for (x = 0; x < 20; x++){
			*p++ |= pgm_read_byte(graph++);
		}
	}

}


//*********************************************************
//	LcdBuff[]からフリップ部分をLCDに転送
//*********************************************************
void trans_flip(void){
	uchar	*p;
	uchar	x, y;

	p = &LcdBuff[(16 / 8) * 101];
	for (y = (16 / 8); y < ((32 + 16) / 8); y++){
		LCD_set(0, y);
		for (x = 0; x < 101; x++){
			LCD_data(*p++);
		}
	}
}


//*********************************************************
//	LcdBuff[]からフリップ部分をクリア
//*********************************************************
void clear_flip(void){
	uchar	*p;
	PGM_VOID_P	graph;
	uchar	x, y;

	p = &LcdBuff[(16 / 8) * 101];
	graph = &GraphData[(16 / 8) * 101];
	for (y = (16 / 8); y < ((32 + 16) / 8); y++){
		for (x = 0; x < 101; x++){
			*p++ = pgm_read_byte(graph++);
		}
	}
}


//*********************************************************
//	LcdBuff[]にグラフィックデータを ORセット(未完成)
//*********************************************************
#if 0
void put_graph(uchar x, uchar y, uchar w, ucahr h, PGM_VOID_P graph){
	uchar	ram_x, ram_y, ram_height;
	uchar	ys, yp;
	uchar	*p;
	//uchar	mask;

	if ((w == 0)||(h == 0))		return;
	if ((x >= 101)||(y >= 64))	return;
	
	ram_height = h / 8;
	ys = y % 8;
	if (h % 8)	ram_height++;
	if (((h % 8) + ys) > 8)	ram_height++;

	if (ram_height == 1){
		//mask = ~(((1<<h) - 1) << (y % 8));
		p = &LcdBuff[(y/8) * 101 + x];
		while(w > 0){
			*p++ = pgm_read_byte(graph) << ys;
			--w;
		}
	} else {
		for (yp = 0; yp < h, yp++){
			if (yp == 0){
			} else if (yp == (h - 1)){
			} else {
			}
		}
	}
}
#endif


//*********************************************************
//	LCD RAMの書き込み開始位置セット
//	xは1ピクセル単位、yは8ピクセル単位
//*********************************************************
void LCD_set(uchar x, uchar y){

	PORTB &= ~((1<<LCDCS)|(1<<LCD_DC));
	SPI_out(0x10 + (x >> 4));
	SPI_out(x & 0x0F);
	SPI_out(0xB0 + y);
	PORTB |= (1<<LCDCS);
}


//*********************************************************
//	LCD消去
//*********************************************************
void clear_LCD(void){
	uchar	x, y;
	uint	i;

	// 画面クリア
	for (y = 0; y < 8; y++){
		LCD_set(0, y);
		for (x = 0; x < 101; x++){
			LCD_data(0);
		}
	}

	for (i = 0; i < 101 * 8; i++){
		LcdBuff[i] = 0;
	}
}


//*********************************************************
//	LCD初期化
//*********************************************************
void LCD_init(void){
	uint	i;

	LCD_RESET_ON;	// LCDリセット
	NOP;
	NOP;
	LCD_RESET_OFF;	// LCDリセット解除

	LCD_command(0xE2);		// reset
	LCD_command(0xA0);		// normal direction
	LCD_command(0xC0);		// normal direction
	LCD_command(0xA2);		// bias
	LCD_command(0x81);		// reference voltage mode
	LCD_command(0x20);

	for (i = 0; i < 2500; i++){	// 適当
		NOP;
	}

	LCD_command(0x2F);		// Power control 内部レギュレータ等、ON
	LCD_command(0x25);		// コントラスト設定抵抗値 1+Rb/Ra = 5.5

	for (i = 0; i < 2500; i++){	// 適当
		NOP;
	}

	clear_LCD();

	LCD_command(0xAF);		// display on

	LCD_contrast(45);		// コントラスト初期値

}


//*********************************************************
//	LCDコントラスト設定
//*********************************************************
void LCD_contrast(uchar contrast){
	if(contrast < 20) contrast = 20;
	if(contrast > 50) contrast = 50;
	PORTB &= ~((1<<LCDCS)|(1<<LCD_DC));
	SPI_out(0x81);
	SPI_out(contrast);
	PORTB |= (1<<LCDCS);
}


//*********************************************************
//	LCD コマンド出力
//*********************************************************
void LCD_command(uchar com){

	PORTB &= ~((1<<LCDCS)|(1<<LCD_DC));
	SPI_out(com);
	PORTB |= (1<<LCDCS);
}


//*********************************************************
//	LCD データ出力
//*********************************************************
void LCD_data(uchar data){

	PORTB &= ~(1<<LCDCS);
	PORTB |= (1<<LCD_DC);
	SPI_out(data);
	PORTB |= (1<<LCDCS);
}


//*********************************************************
//	A/Dコンバータデータ入力
// ch :		A/Dチャンネル
// 戻り値:	A/D変換値(0〜1023)
//*********************************************************
uint ADCin(char ch){
	uint	val;

	ADMUX = (1<<REFS1)|(1<<REFS0)|ch;	// 内部基準電圧 1.1V, A/D ch
	ADCSRA = (1<<ADEN)|(1<<ADPS1)|(1<<ADPS0);	// A/D変換回路ON プリスケーラ 1MHz/8 = 125kHz

	ADCSRA |= (1<<ADSC);			// 変換開始

	while(!(ADCSRA&(1<<ADIF)));		// ADIF=1 になるまでループ
	ADCSRA |= (1<<ADIF);			// ADIFクリア(1を書くとクリアされる)

	val = (uint)ADCL;	
	return (((uint)ADCH)<<8)|val;
}


