//******************************************************************
//
//	パスワールド液晶時計
//
//	DEVICE:ATmega168
//	efuse:F9h	
//	hfuse:D6h	BOD 1.8V	チップ消去から EEPROM保護
//	lfuse:62h	内蔵CR発振 1MHz
//
//	Compiler:	AVR-GCC		3.4.6
//				AVR-LIBC	1.4.6
//				binutils	2.16.1
//
//	2007.10.14	KAWAKAMI Yukio
//******************************************************************

#define		LF	10
#define		CR	13
#define		TAB	9
#define		ESC	27
#define		ON	1
#define		OFF	0
//#define		TRUE	1
//#define		FALSE	0

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;
typedef unsigned short	ushort;

#define	SPI_CLK		5	// PORTB
#define	SPI_DIN		4
#define	SPI_DOUT	3
#define	LCD_DC		2
#define	LCDCS		1
#define	LCDRS		0

#define	SCL			5	// PORTC
#define	SDA			4

#define	JJY_DATA	3
#define	JJY_PON		2

#define	SW2			4	// PORTD
#define	SW1			3
#define	CLKIN		2

#define		JJY_ON			(!(PINC & (1<<JJY_DATA)))	// SM9502Aは負論理
#define		JJY_ENABLE		PORTC &= ~(1<<JJY_PON); DDRC |= (1<<JJY_PON)
#define		JJY_DISABLE		DDRC &= ~(1<<JJY_PON); PORTC |= (1<<JJY_PON)
#define		CHECK_JJY_ON	(!(PORTC & (1<<JJY_PON)))

#define		CHECK_SW1		(!(PIND & (1<<SW1)))
#define		CHECK_SW2		(!(PIND & (1<<SW2)))

#define		LCD_RESET_ON	PORTB &= ~(1<<LCDRS)
#define		LCD_RESET_OFF	PORTB |= (1<<LCDRS)

#define		EEPROM_ADJUST_HOUR		1
#define		EEPROM_ADJUST_MIN		2
#define		EEPROM_CONTRAST			3

struct RTC_TIME {
	uchar	year;
	uchar	month;
	uchar	week;
	uchar	day;
	uchar	hour;
	uchar	min;
	uchar	sec;
};

void read_RTC(struct RTC_TIME *rtc);
void adjust_RTC(struct RTC_TIME *rtc);

char	TWI_open(void);
char	TWI_start(void);
char	TWI_write(uchar data);
uchar	TWI_read_ack(void);
uchar	TWI_read_nack(void);
void	TWI_close(void);

uint	ADCin(char ch);

void print(uchar x, uchar y, uchar *str);
void print_P(uchar x, uchar y, PGM_VOID_P str);
void print_shift(uchar x, uchar y, uchar *str);
void print_shift_P(uchar x, uchar y, PGM_VOID_P str);
void print_num_right(uint num, uchar x, uchar y, uchar w);
void print_num_zero(uint num, uchar x, uchar y, uchar w);

void clear_LCD(void);
void LCD_contrast(uchar contrast);

void	LCD_command(uchar com);
void	LCD_data(uchar data);
void	LCD_init(void);

void	LCD_set(uchar x, uchar y);

uchar	SPI_in(void);
void	SPI_out(uchar data);

void next_min(struct RTC_TIME *rtc);
void JJY2RTC(struct RTC_TIME *rtc);

uchar JJY_adjusting(void);
void start_adjust(void);

uchar BIN(uchar bcd);
uchar BCD(uchar bin);
uchar add_BCD(uchar source, uchar add);

void put_wallpaper(PGM_VOID_P graph);
void put_graph_num(uchar xs, PGM_VOID_P graph);
void trans_flip(void);
void clear_flip(void);

void wait_switch_off(void);

void menu(void);
void adjust_clock(void);
uchar adjust_clock_sub(uchar x, uchar y, uchar *n, uchar min, uchar max);
void view_adjust_time(void);
void set_adjust_time(void);
void broken_demo(void);


